DECLARE FUNCTION LinearBlend! (t!, N1!, N2!)
DECLARE FUNCTION QuadraticBlend! (t!, N1!, N2!, P1!)
DECLARE FUNCTION CubicBlend! (t!, N1!, N2!, P1!, P2!)
DECLARE FUNCTION HermiteBlend! (t!, N1!, N2!, P1!, P2!)
' BLEND1.BAS
' By Dan Farmer
' This program demonstrates four different methods for 
' interpolating between two values. 
CLS
f$ = "#.# ]  #.##"
N1 = 1
N2 = 2
P1 = 1.25
P2 = 1

LOCATE 1, 3
PRINT USING " [N1=" + f$; N1;
PRINT USING " [N2=" + f$; N2;
PRINT USING " [P1=" + f$; P1;
PRINT USING " [P2=" + f$; P2

' Linear interpolation
col = 1
astep = .1
LOCATE 3, col
PRINT "Linear"
LOCATE CSRLIN, col: PRINT "------"
FOR t = 0 TO 1 + astep STEP astep
    n = LinearBlend(t, N1, N2)
    LOCATE CSRLIN, col: PRINT USING f$; t; n
NEXT t

' Quadratic interpolation
col = 20
LOCATE 3, col
PRINT "Quadratic"
LOCATE CSRLIN, col: PRINT "-----"
FOR t = 0 TO 1 + astep STEP astep
    n = QuadraticBlend(t, N1, N2, P1)
    LOCATE CSRLIN, col: PRINT USING f$; t; n
NEXT t

' Cubic interpolation
col = 40
LOCATE 3, col
PRINT "Cubic"
LOCATE CSRLIN, col: PRINT "-----"
FOR t = 0 TO 1 + astep STEP astep
    n = CubicBlend(t, N1, N2, P1, P2)
    LOCATE CSRLIN, col: PRINT USING f$; t; n
NEXT t

' Hermite interpolation
col = 60
LOCATE 3, col
PRINT "Hermite"
LOCATE CSRLIN, col: PRINT "-----"
FOR t = 0 TO 1 + astep STEP astep
    n = HermiteBlend(t, N1, N2, P1, P2)
    LOCATE CSRLIN, col: PRINT USING f$; t; n
NEXT t

FUNCTION CubicBlend (t, N1, N2, P1, P2) STATIC
    CubicBlend = (1 - t) ^ 3 * N1 + (t ^ 3) * N2 + 3 * t * ((1 - t) ^ 2) * P1 + 3 * (t ^ 2) * (1 - t) * P2
END FUNCTION

FUNCTION LinearBlend (t, N1, N2) STATIC
    LinearBlend = (1 - t) * N1 + t * N2
END FUNCTION

FUNCTION QuadraticBlend (t, N1, N2, P1) STATIC
    QuadraticBlend = (1 - t) ^ 2 * N1 + t ^ 2 * N2 + 2 * t * (1 - t) * P1
END FUNCTION

FUNCTION HermiteBlend(t, N1,N2,P1,P2)
    HermiteBlend = (N1*((2.0*t-3.0)*t^2+1.0)+N2*(-2.0*t+3.0)*t^2+P1*((t-2.0)*t+1.0)*t+P2*(t-1.0)*t^2 )
END FUNCTION    
