DECLARE FUNCTION SCurve! (t!)
DECLARE FUNCTION GMean! (x!)
DECLARE FUNCTION HermiteBlend! (t!, N1!, N2!, P1!, P2!)
DECLARE FUNCTION LinearBlend! (t!, N1!, N2!)
DECLARE FUNCTION QuadraticBlend! (t!, N1!, N2!, P1!)
DECLARE FUNCTION CubicBlend! (t!, N1!, N2!, P1!, P2!)
'-----------------------------------------------------------------------
' BLEND2.BAS
' By Dan Farmer
' This program demonstrates graphically four different methods for
' interpolating between two values.
' P1 is a slope control parameter for Quadratic, Cubic, and Hermite blends.
' P2 is a second slope control parameter for Cubic and Hermite blends.
'-----------------------------------------------------------------------
SCREEN 12: MAXX = 640: MAXY = 480: MAXCOLORS = 16

incinc = .01                           ' timestep increment
increment = incinc * 10
mininc = increment * .05               ' lower stop for incr control
maxinc = increment * 2                 ' upper stop for incr control


x1 = 0: x2 = 1
y1 = 1.8: y2 = 4
border = 1

' --- Set initial values for the curve controls
N1 = 0: N2 = 1
P1 = 0: P2 = 0

WinBG = 8: WinBorder = 16
TicColor = 7

maxtypes = 3                           ' Number of curve types to display

' Set up a color table, one entry for each curve type
' Should be kept to the lower 7 colors because the circle
' are drawn with ColorTable(n)+8 for a darker outline.
' (Got a couple of unused ones set up here, too)
DIM ColorTable(7)
ColorTable(1) = 2
ColorTable(2) = 3
ColorTable(3) = 4
ColorTable(4) = 5
ColorTable(5) = 6
ColorTable(6) = 7


DO

    ' Predraw the windows... purely cosmetic. 
    GOSUB SetGraphVP
    GOSUB SetAnimVP
    
    ' Once for the graph, once for the "animation"
    FOR j% = 1 TO 2
    
        IF j% = 1 THEN
            GOSUB SetGraphVP
        ELSE
            GOSUB SetAnimVP
        END IF
    
        ' --- Loop through all types and graph each.
        FOR i% = 1 TO maxtypes
        
            IF j% = 1 THEN PSET (x1, y1)   ' Anchor starting point for lineto
            
            FOR t = 0 TO 1 + increment STEP increment
                SELECT CASE i%
                CASE 1
                    n = QuadraticBlend(t, N1, N2, P1)
                    k$ = "Quadratic(t,N1,N2,P1)"
                CASE 2
                    n = CubicBlend(t, N1, N2, P1, P2)
                    k$ = "Cubic (t,N1,N2,P1,P2)"
                CASE 3
                    n = HermiteBlend(t, N1, N2, P1, P2)
                    k$ = "Hermite (t,N1,N2,P1,P2)"
                CASE 4                 ' Unused
                    n = 0
                    k$ = ""
                CASE ELSE
                END SELECT
                IF j% = 1 THEN
                    GOSUB Graph
                ELSE
                    GOSUB Animate
                END IF
    
                ' Print the name of the method in it's proper color
                COLOR ColorTable(i%)
                LOCATE 1 + keytop + i%, keycol: PRINT k$
                
                k$ = INKEY$
                IF k$ = CHR$(27) THEN END
                
            NEXT t
        NEXT i%
    NEXT j%
    
    ' --- Diplay parms, get user input, crudely.
    DO WHILE (1)
        COLOR 3
        ' Display current values
        LOCATE 1, 50: PRINT USING "[P1 (lower slope) =###.##]"; P1
        LOCATE 2, 50: PRINT USING "[P2 (upper slope) =###.##]"; P2
        LOCATE 3, 50: PRINT USING "[Time Step = #.###"; increment
        LOCATE 5, 50: PRINT "[1 - 2] = Change P1 (-/+)"
        LOCATE 6, 50: PRINT "[3 - 4] = Change P2 (-/+)"
        LOCATE 7, 50: PRINT "[5 - 6] = Change Incr(-/+)"
        LOCATE 8, 50: PRINT "[ESC] = Quit"
        LOCATE 9, 50: PRINT "[ENTER] = Redraw"

        ' User control: check keypress
        which$ = INPUT$(1)
        SELECT CASE which$
        CASE CHR$(27)
            END
        CASE "1"
            P1 = P1 - .1
        CASE "2"
            P1 = P1 + .1
        CASE "3"
            P2 = P2 - .1
        CASE "4"
            P2 = P2 + .1
        CASE "5"
            increment = increment - incinc
            IF increment < mininc THEN increment = mininc
        CASE "6"
            increment = increment + incinc
            IF increment > maxinc THEN increment = maxinc
        CASE ELSE
            EXIT DO
        END SELECT
        
    LOOP
    CLS
LOOP WHILE (1)
END

SetGraphVP:

    ' Set up Graph viewport
    VIEW (1, 1)-(MAXX / 2, MAXY / 2), WinBG, WinBorder
    WINDOW (0, N1)-(1, N2)

    ' Draw tics
    IF N2 > N1 THEN ydist = N2 - N1 ELSE ydist = N1 - N2
    yincr = ydist / 10

    FOR y = N1 TO N2 + yincr STEP yincr
        LINE (0, y)-(1, y), TicColor
    NEXT y
   
    FOR x = x1 TO x2 + increment STEP increment
        LINE (x, N1)-(x, 1), TicColor
    NEXT x

    COLOR TicColor
    LOCATE 17, 1: PRINT "0"
    LOCATE 17, 40: PRINT "1"
    LOCATE 17, 16: PRINT "T i m e"

    LOCATE 1, 42: PRINT LTRIM$(STR$(N2));
    LOCATE 5, 42: PRINT "V"
    LOCATE 6, 42: PRINT "a"
    LOCATE 7, 42: PRINT "l"
    LOCATE 8, 42: PRINT "u"
    LOCATE 9, 42: PRINT "e"
    LOCATE 16, 42: PRINT LTRIM$(STR$(N1))
  
    ' --- Show a color key
    keytop = 12: keycol = 50
    COLOR 15: LOCATE keytop, keycol: PRINT "Key: ";
    COLOR 7: LOCATE keytop + 1, keycol: PRINT "";

RETURN

SetAnimVP:
    ' Set up Animation viewport
    ytop = (MAXY / 2) + 40: ybottom = MAXY - 20
    xleft = 1: xright = MAXX - 2

    VIEW (xleft, ytop)-(xright, ybottom), WinBG, WinBorder
    WINDOW (xleft, ytop)-(xright - 2, ybottom)

    FOR i% = xleft TO xright STEP (xright - xleft) / 10
        LINE (i%, ytop)-(i%, ybottom), TicColor
    NEXT i%

    FOR i% = 1 TO 5
        yp = ytop + ((ybottom - ytop) / (maxtypes + 1)) * i%
        LINE (xleft, yp)-(xright, yp), TicColor
    NEXT i%

RETURN
    
Graph:
    IF increment < .005 THEN
        PSET (t, n), ColorTable(i%)
    ELSE
        LINE -(t, n), ColorTable(i%)
    END IF
RETURN

Animate:
    yd = ybottom - ytop
    rad = 5
    xp = rad + n * MAXX
    yp = ytop + (yd / (maxtypes + 1)) * i%
    c = ColorTable(i%)
    CIRCLE (xp, yp), rad, c
    PAINT (xp, yp), c
    CIRCLE (xp, yp), rad, c + 8
RETURN

FUNCTION CubicBlend (t, N1, N2, P1, P2) STATIC
    CubicBlend = (1 - t) ^ 3 * N1 + (t ^ 3) * N2 + 3 * t * ((1 - t) ^ 2) * P1 + 3 * (t ^ 2) * (1 - t) * P2
END FUNCTION

FUNCTION GMean (x) STATIC
    GMean = x * ((SQR(5) + 1) / 2)
END FUNCTION

FUNCTION HermiteBlend (t, N1, N2, P1, P2)
    HermiteBlend = (N1 * ((2! * t - 3!) * t ^ 2 + 1!) + N2 * (-2! * t + 3!) * t ^ 2 + P1 * ((t - 2!) * t + 1!) * t + P2 * (t - 1!) * t ^ 2)
END FUNCTION

FUNCTION LinearBlend (t, N1, N2) STATIC
    LinearBlend = (1 - t) * N1 + t * N2
END FUNCTION

FUNCTION QuadraticBlend (t, N1, N2, P1) STATIC
    QuadraticBlend = (1 - t) ^ 2 * N1 + (t ^ 2) * N2 + 2 * t * (1 - t) * P1
END FUNCTION

FUNCTION SCurve (t) STATIC
    SCurve = (t * t) * (3 - (2 * t))
END FUNCTION

