DECLARE SUB Get.Data (A$)
DECLARE SUB Read.Data (A$, x!, y!, z!)
DECLARE SUB READKEY (KEYPRESS!)
DECLARE SUB Perspective (x!, y!, z!, sx!, sy!)
' Notes: currently requires the TRIANGLE statement to be all in one line
'        i.e.: TRIANGLE <x y z> <x y z> <x y z> END_TRIANGLE
'        Problems with "Wait... Ready" messages
'        Calling display twice is TOO slow!

COMMON SHARED rho, d, c1, s1, c2, s2, theta, phi
CONST FALSE = 0
CONST TRUE = NOT FALSE

'---  STRIP LEADING BLANK FROM "STR$"
DEF FNSTNG$ (x!) = MID$(STR$(x!), 2)

DEF fnUnPad$ (x$)
	WHILE LEFT$(x$, 1) = " "
		x$ = MID$(x$, 2)
	WEND
	WHILE RIGHT$(x$, 1) = " "
	   x$ = LEFT$(x$, LEN(x$) - 1)
	WEND
	fnUnPad$ = x$
END DEF

MAXDATA = 1023
DIM buffer(MAXDATA, 3, 3)

rho = 30      ' distance, viewer to object origin /default=30
d = 10        ' distance from viewer screen plane /default=10
theta = .3     ' viewer angle 1 (rotate left-right) /default=.3
phi = 1       ' viewer angle 2 (something like rotate up/down)  /default=1
s1 = SIN(theta)
c1 = COS(theta)
s2 = SIN(phi)
c2 = COS(phi)
apage = 1: vpage = apage
x.vector = 1: y.vector = 2: z.vector = 3

infile$ = "pyramid.dat"
'INFILE$ = "c:\dl\fract001.dat"

OPEN infile$ FOR INPUT AS #1
LOCATE 1.1: PRINT "Reading data :"

MAIN:
	DO WHILE NOT EOF(1)
		CALL Get.Data(d$)
		IF INSTR(d$, "TRIANGLE") = 0 THEN GOTO MAIN
		triangle.count = triangle.count + 1
		LOCATE 1, 15: PRINT USING "#### triangles"; triangle.count
		FOR vector = x.vector TO z.vector 'Draw one triangle (with 3 sets of vectors)
			CALL Read.Data(d$, x, y, z)
			buffer(triangle.count, vector, x.vector) = x
			buffer(triangle.count, vector, y.vector) = y
			buffer(triangle.count, vector, z.vector) = z
		NEXT vector
	LOOP
	akey = 0
DO

	SELECT CASE akey
		CASE 72                                  ' UP
			phi = phi + .5
		CASE 80                                  ' DOWN
			phi = phi - .5
		CASE 77                                  ' RIGHT
			theta = theta + .5
		CASE 75                                  ' LEFT
			theta = theta - .5
		CASE 71                                  ' HOME (MOVE rho CLOSER)
			rho = rho * .75
		CASE 79                                  ' END ( MOVE rho BACK )
			rho = rho / .75
		CASE 73                                  ' PAGE UP (MOVE d CLOSER)
			d = d / .75
		CASE 81                                  ' PAGE DN (MOVE d BACK)
			d = d * .75
		CASE 27
			EXIT DO                              ' QUIT
		CASE ELSE
			LOCATE 1, 1: PRINT "Ready"
			BEEP                                 ' FROWN
	END SELECT

	s1 = SIN(theta)
	c1 = COS(theta)
	s2 = SIN(phi)
	c2 = COS(phi)
   
	CALL READKEY(akey)
	'CALL Display

LOOP
END.MAIN:
SYSTEM

SUB Display (buffer(), triangle.count) STATIC
STATIC oldtheta, delta.theta
	SCREEN 8
	CLS 2
	VIEW (180, 20)-(620, 180), , 3
	WINDOW (-4, -3)-(4, 3)
	PSET (0, 0), 0


	FOR I% = 1 TO triangle.count
		FOR vector = x.vector TO z.vector
			x = buffer(I%, vector, x.vector)
			y = buffer(I%, vector, y.vector)
			z = buffer(I%, vector, z.vector)
			kolor% = 4
			CALL Perspective(x, y, z, sx, sy)
			IF vector = 1 THEN
				PSET (sx, sy), kolor%
				start.point.x = sx
				start.point.y = sy
			END IF
			LINE -(sx, sy), kolor%
		NEXT vector
		LINE -(start.point.x, start.point.y), kolor%      ' complete the connetction
	NEXT I%
END SUB

SUB Get.Data (A$) STATIC
	DO
	INPUT #1, A$
	LOOP WHILE INSTR(A$, "TRIANGLE") = 0 AND NOT EOF(1)
END SUB

SUB Perspective (x, y, z, sx, sy) STATIC
' COMMON SHARED by calling program:  These variables MUST be defined.
' rho   :=  distance, viewer to object origin
' d     :=  distance from viewer screen plane
' s1,c1 :=  SIN/COS(theta) respectively.  Viewer angle 1
' s2,c2 :=  SIN/COS(phi)   respectively.  Viewer angle 2
'
' Parameters:
' Input:
' x,y,z :=  points to be modified by perspective distortion
'
' Output:
' sx,sy := new adjusted screen coordinates. Plot these points.
'
STATIC xe, ye, ze
	xe = -x * s1 + z * c1
	ze = -x * c1 * c2 - z * s1 * c2 + y * s2
	ye = -x * s2 * c1 - z * s1 * s2 - y * c2 + rho
	sx = d * xe / ye
	sy = d * ze / ye
END SUB

SUB Read.Data (A$, x, y, z) STATIC
STATIC I, Bracket1, Bracket2, vector$

	Bracket1 = INSTR(A$, "<")
	Bracket2 = INSTR(A$, ">")
	vector$ = fnUnPad$(MID$(A$, Bracket1 + 1, Bracket2 - Bracket1 - 1))
	A$ = fnUnPad$(MID$(A$, Bracket2 + 1))
	aspace = INSTR(vector$, " ")
	x = VAL(LEFT$(vector$, aspace))
	vector$ = fnUnPad$(MID$(vector$, aspace))
	aspace = INSTR(vector$, " ")
	y = VAL(LEFT$(vector$, aspace))
	vector$ = fnUnPad$(MID$(vector$, aspace))
	z = VAL(vector$)
END SUB

' Returns one-byte VALUE of KEYPRESS
SUB READKEY (KEYPRESS) STATIC
	CURSORUP$ = CHR$(0) + CHR$(72)
	CURSORLEFT$ = CHR$(0) + CHR$(75)
	CURSORDOWN$ = CHR$(0) + CHR$(80)
	CURSORIGHT$ = CHR$(0) + CHR$(77)
	PAGEUP$ = CHR$(0) + CHR$(73)
	PAGEDN$ = CHR$(0) + CHR$(81)
	HOME$ = CHR$(0) + CHR$(71)
	END$ = CHR$(0) + CHR$(79)
	CRETURN$ = CHR$(13)
	ESCAPE$ = CHR$(27)

		READCURSOR$ = ""
		WHILE READCURSOR$ <> CURSORUP$ AND READCURSOR$ <> CURSORIGHT$ AND READCURSOR$ <> CURSORDOWN$ AND READCURSOR$ <> CURSORLEFT$ AND READCURSOR$ <> PAGEUP$ AND READCURSOR$ <> PAGEDN$ AND READCURSOR$ <> HOME$ AND READCURSOR$ <> END$ AND READCURSOR$ <> CRETURN$ AND READCURSOR$ <> ESCAPE$
			READCURSOR$ = INKEY$
		 WEND
		 KEYPRESS = ASC(RIGHT$(READCURSOR$, 1))
END SUB

